/* Project name:
     Thumb wheel click
 * Copyright:
     (c) Mikroelektronika, 2016.
 * Revision History:
     - mikroC PRO(AM);
 * Description:
     Each DS2408 IC has a factory-lasered 64-bit registration number.
     Combined with the standard Dallas Semiconductor 1-Wire protocol it uses to communicate with the target board MCU,
     the registration number enables multiple devices to reside on the same data line, each one with a unique ID.
     To connect multiple Thumb Wheel clicks, use the additional three pins positioned above the mikroBUS socket (VCC, OW, GND).

 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7 - ac:ADC
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.0000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
       - Place Thumb Wheel click to the mikroBUS1 socket
       - Turn on LED's SW15.2 and SW15.3

/* Pins
 ******************************************************************************/
sbit DS2408_RST at LATC1_bit;
sbit DS2408_RST_DIR at TRISC1_bit;

/* Constants
 *******************************************************************************/

// Register Addresses
#define DS2408_PIO_LOGIC_REG       0x0088
#define DS2408_PIO_OUTPUT_REG      0x0089
#define DS2408_PIO_ACTIVITY_REG    0x008A
#define DS2408_SEARCH_MASK_REG     0x008B
#define DS2408_SEARCH_SELECT_REG   0x008C
#define DS2408_CONTROL_STATUS_REG  0x008D

// Commands
#define DS2408_PIO_READ_CMD        0xF0
#define DS2408_CHANNEL_READ_CMD    0xF5
#define DS2408_CHANNEL_WRITE_CMD   0x5A
#define DS2408_RESET_CMD           0xC3
#define DS2408_SKIP_ROM_CMD        0xCC

/* Global Variables
 ******************************************************************************/
int tmp;

/* Set Configuration Registers
 ******************************************************************************/
void DS2408_SetConfig( char cfg )
{
    while( Ow_Reset( &PORTD, 0 ) );

    Ow_Write( &PORTD, 0, DS2408_SKIP_ROM_CMD );                            // One-Wire write data to RC0 pin
    Ow_Write( &PORTD, 0, DS2408_SKIP_ROM_CMD );
    Ow_Write( &PORTD, 0, DS2408_CONTROL_STATUS_REG );
    Ow_Write( &PORTD, 0, DS2408_CONTROL_STATUS_REG >> 8 );
    Ow_Write( &PORTD, 0, cfg );
}

/* Software Reset DS2408
 ******************************************************************************/
void DS2408_SoftReset()
{
    while( Ow_Reset( &PORTD, 0 ) );

    Ow_Write( &PORTD, 0, DS2408_SKIP_ROM_CMD );
    Ow_Write( &PORTD, 0, DS2408_RESET_CMD );
}

/* Read DS2408
 ******************************************************************************/
char DS2408_ChannelRead()
{
    char state;

    while( Ow_Reset( &PORTD, 0 ) );                                        // Reset pulse and wait presence

    Ow_Write( &PORTD, 0, DS2408_SKIP_ROM_CMD );                            // Issue skip ROM command
    Ow_Write( &PORTD, 0, DS2408_CHANNEL_READ_CMD );                        // Issue channel-access write command
    state = Ow_Read( &PORTD, 0 );                                          // Write byte to PIO

    return state;
}

/* Display result on LED's
 ******************************************************************************/
void LED_on( int pos )
{
     LATB = 0;

     do
     {
         LATB |= 1 << pos;
     }
     while( pos-- );
}

/* Main function
 ******************************************************************************/
void main()
{
    DS2408_RST_DIR = 0;
    TRISB          = 0x00;
    Delay_ms( 100 );

    DS2408_SoftReset();
    Delay_ms( 100 );

    DS2408_SetConfig( 0x01 );
    DS2408_RST = 1;
    Delay_ms( 200 );

    while( 1 )
    {
        tmp = DS2408_ChannelRead();                                             // Read DS2408
        LED_on( tmp );
    }
}